<?php

namespace Drupal\easy_entity_field\Entity;

use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Provides an interface for defining Easy entity field entities.
 */
interface EasyEntityFieldInterface extends ConfigEntityInterface {

  /**
   * Returns the machine name of the field.
   *
   * This defines how the field data is accessed from the entity. For example,
   * if the field name is "foo", then $entity->foo returns its data.
   *
   * @return string
   *   The field name.
   */
  public function getName();

  /**
   * Returns the field type.
   *
   * @return string
   *   The field type, i.e. the id of a field type plugin. For example 'text'.
   *
   * @see \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  public function getFieldType();

  /**
   * Returns a Field label.
   *
   * @return string|\Drupal\Core\StringTranslation\TranslatableMarkup|null
   *   The label. A string or an instance of TranslatableMarkup will be returned
   *   based on the way the label translation is handled. NULL if no label is
   *   available.
   */
  public function getLabel();

  /**
   * Sets the Field label.
   *
   * @param string $label
   *   The label to set.
   *
   * @return $this
   */
  public function setLabel($label);

  /**
   * Returns the Field description.
   *
   * @return string
   *   The Field description.
   */
  public function getDescription();

  /**
   * Sets the Field description.
   *
   * @param string $description
   *   The Field description.
   *
   * @return $this
   */
  public function setDescription($description);

  /**
   * Returns whether the field can be empty.
   *
   * If a field is required, an entity needs to have at least a valid,
   * non-empty item in that field's FieldItemList in order to pass validation.
   *
   * An item is considered empty if its isEmpty() method returns TRUE.
   * Typically, that is if at least one of its required properties is empty.
   *
   * @return bool
   *   TRUE if the field is required.
   *
   * @see \Drupal\Core\TypedData\Plugin\DataType\ItemList::isEmpty()
   * @see \Drupal\Core\Field\FieldItemInterface::isEmpty()
   * @see \Drupal\Core\TypedData\DataDefinitionInterface:isRequired()
   * @see \Drupal\Core\TypedData\TypedDataManager::getDefaultConstraints()
   */
  public function isRequired();

  /**
   * Sets whether the field can be empty.
   *
   * If a field is required, an entity needs to have at least a valid,
   * non-empty item in that field's FieldItemList in order to pass validation.
   *
   * An item is considered empty if its isEmpty() method returns TRUE.
   * Typically, that is if at least one of its required properties is empty.
   *
   * @param bool $required
   *   TRUE if the field is required. FALSE otherwise.
   *
   * @return $this
   *   The current object, for a fluent interface.
   */
  public function setRequired($required);

  /**
   * Gets the plugin for the field type.
   *
   * @return mixed
   */
  public function getFieldPlugin();

  /**
   * Plugin for verifying field types.
   *
   * @return mixed
   */
  public function isFieldPlugin();

  /**
   * Get all field settings and field storage settings.
   *
   * @return mixed
   */
  public function getSettings();

  /**
   * Get all field settings.
   *
   * @return mixed
   */
  public function getFieldSettings();

  /**
   * Get all field storage settings.
   *
   * @return mixed
   */
  public function getStorageSettings();

  /**
   * Gets the value set for a certain field.
   *
   * @param $setting_name
   *
   * @return mixed
   */
  public function getFieldSetting($setting_name);

  /**
   * Gets the value of a field storage setting.
   *
   * @param $setting_name
   *
   * @return mixed
   */
  public function getStorageSetting($setting_name);

  /**
   * Sets field settings.
   *
   * @param array $settings
   *   The array of field settings.
   *
   * @return $this
   */
  public function setFieldSettings(array $settings);

  /**
   * Sets field Storage settings.
   *
   * @param array $settings
   *   The array of field settings.
   *
   * @return $this
   */
  public function setStorageSettings(array $settings);

  /**
   * Returns the default value for the field in a newly created entity.
   *
   * This method computes the runtime default value for a field in a given
   * entity. To access the raw properties assigned to the field definition,
   * ::getDefaultValueLiteral() or ::getDefaultValueCallback() should be used
   * instead.
   *
   * @return array
   *   The default value for the field, as a numerically indexed array of items,
   *   each item being a property/value array (array() for no default value).
   *
   * @see FieldDefinitionInterface::getDefaultValueLiteral()
   * @see FieldDefinitionInterface::getDefaultValueCallback()
   */
  public function getDefaultValue();

  /**
   * Sets the default data value.
   *
   * @param array $values
   *
   * @return $this
   */
  public function setDefaultValue(array $values);

  /**
   * Returns the field storage definition.
   *
   * @return \Drupal\Core\Field\FieldStorageDefinitionInterface
   *   The field storage definition.
   */
  public function getFieldStorageDefinition();

  /**
   * Gets the current display plugin.
   *
   * @param string $display_name
   *   The current display plugin.
   */
  public function getDisplay(string $display_name);

  /**
   * Determines if the storage contains any data.
   *
   * @return bool
   *   TRUE if the storage contains data, FALSE if not.
   */
  public function hasData();

  /**
   * Sets the ID of the type of the entity this field is attached to.
   *
   * @param string $entity_type_id
   *   The name of the target entity type to set.
   *
   * @return $this
   */
  public function setTargetEntityTypeId($entity_type_id);

  /**
   * Gets the entity type for which this display is used.
   *
   * @return string
   *   The entity type id.
   */
  public function getTargetEntityTypeId();

  /**
   * Gets the entity type for which this display is used.
   *
   * @return string
   *   The entity type.
   */
  public function getTargetEntityType();

  /**
   * Verify the entity type using this display.
   *
   * @return mixed
   */
  public function isTargetEntityType();

  /**
   * Returns the route parameters needed to render a link for the action.
   *
   * @param $parameter
   *
   * @return mixed
   */
  public function getRouteParameter($parameter);

  /**
   * Gets the base field definition.
   *
   * @return \Drupal\Core\Field\BaseFieldDefinition
   */
  public function getBaseFieldDefinition();

  /**
   * Builds base field definitions for an entity type.
   *
   * @return \Drupal\Core\Field\FieldDefinitionInterface[]
   *   An array of field definitions, keyed by field name.
   *
   * @throws \LogicException Thrown if a config entity type is given or if one of the entity keys is
   *   flagged as translatable.
   */
  public function buildBaseFieldDefinitions();

}
